@@ -1,6 +1,3 @@
-# coding: utf-8
-from __future__ import absolute_import, division, print_function
-
 from .base import (  # noqa
     AuthenticationError, SpaceTrackClient, UnknownPredicateTypeWarning)
 from .operators import (  # noqa
@@ -17,7 +14,7 @@ __all__ = (
     'startswith',
 )
 
-__version__ = '0.13.7'
+__version__ = '0.15.0'
 __description__ = 'Python client for space-track.org'
 
 __license__ = 'MIT'
