@@ -1,27 +1,23 @@
-# coding: utf-8
-from __future__ import absolute_import, division, print_function
-
 import datetime as dt
 import re
 import threading
 import time
 import warnings
 import weakref
+from collections.abc import Mapping
 from collections import OrderedDict
 from functools import partial
 
 import requests
 from logbook import Logger
-from ratelimiter import RateLimiter
 from represent import ReprHelper, ReprHelperMixin
+from rush.quota import Quota
+from rush.throttle import Throttle
+from rush.limiters.periodic import PeriodicLimiter
+from rush.stores.dictionary import DictionaryStore as RushDictionaryStore
 
 from .operators import _stringify_predicate_value
 
-try:
-    from collections.abc import Mapping
-except ImportError:
-    from collections import Mapping
-
 logger = Logger('spacetrack')
 
 type_re = re.compile(r'(\w+)')
@@ -36,6 +32,8 @@ enum_re = re.compile(r"""
 
 BASE_URL = 'https://www.space-track.org/'
 
+RATELIMIT_KEY = 'request'
+
 
 class AuthenticationError(Exception):
     """Space-Track authentication error."""
@@ -45,7 +43,7 @@ class UnknownPredicateTypeWarning(RuntimeWarning):
     """Used to warn when a predicate type is unknown."""
 
 
-class Predicate(ReprHelperMixin, object):
+class Predicate(ReprHelperMixin):
     """Hold Space-Track predicate information.
 
     The current goal of this class is to print the repr for the user.
@@ -83,7 +81,7 @@ class Predicate(ReprHelperMixin, object):
             return value
 
 
-class SpaceTrackClient(object):
+class SpaceTrackClient:
     """SpaceTrack client class.
 
     Parameters:
@@ -146,6 +144,8 @@ class SpaceTrackClient(object):
         'tip',
         'announcement',
         'gp',
+        'gp_history',
+        'cdm_public',
     }
 
     request_controllers['expandedspacedata'] = {
@@ -207,14 +207,23 @@ class SpaceTrackClient(object):
         self._predicates = dict()
         self._controller_proxies = dict()
 
-        # "Space-track throttles API use in order to maintain consistent
-        # performance for all users. To avoid error messages, please limit
-        # your query frequency to less than 20 requests per minute."
-        self._ratelimiter = RateLimiter(
-            max_calls=19, period=60, callback=self._ratelimit_callback)
+        # From https://www.space-track.org/documentation#/api:
+        #   Space-track throttles API use in order to maintain consistent
+        #   performance for all users. To avoid error messages, please limit
+        #   your query frequency.
+        #   Limit API queries to less than 30 requests per minute / 300 requests
+        #   per hour
+        self._per_minute_throttle = Throttle(
+            limiter=PeriodicLimiter(RushDictionaryStore()),
+            rate=Quota.per_minute(30),
+        )
+        self._per_hour_throttle = Throttle(
+            limiter=PeriodicLimiter(RushDictionaryStore()),
+            rate=Quota.per_hour(300),
+        )
 
     def _ratelimit_callback(self, until):
-        duration = int(round(until - time.time()))
+        duration = int(round(until - time.monotonic()))
         logger.info('Rate limit reached. Sleeping for {:d} seconds.', duration)
 
         if self.callback is not None:
@@ -327,12 +336,10 @@ class SpaceTrackClient(object):
         else:
             classes = self.request_controllers.get(controller, None)
             if classes is None:
-                raise ValueError(
-                    'Unknown request controller {!r}'.format(controller))
+                raise ValueError(f'Unknown request controller {controller!r}')
             if class_ not in classes:
                 raise ValueError(
-                    'Unknown request class {!r} for controller {!r}'
-                    .format(class_, controller))
+                    f'Unknown request class {class_!r} for controller {controller!r}')
 
         # Decode unicode unless class == download, including conversion of
         # CRLF newlines to LF.
@@ -346,8 +353,7 @@ class SpaceTrackClient(object):
 
         self.authenticate()
 
-        url = ('{0}{1}/query/class/{2}'
-               .format(self.base_url, controller, class_))
+        url = f'{self.base_url}{controller}/query/class/{class_}'
 
         offline_check = (class_, controller) in self.offline_predicates
         valid_fields = {p.name for p in self.rest_predicates}
@@ -364,16 +370,14 @@ class SpaceTrackClient(object):
 
         for key, value in kwargs.items():
             if key not in valid_fields:
-                raise TypeError(
-                    "'{class_}' got an unexpected argument '{key}'"
-                    .format(class_=class_, key=key))
+                raise TypeError(f"'{class_}' got an unexpected argument '{key}'")
 
             if class_ == 'upload' and key == 'file':
                 continue
 
             value = _stringify_predicate_value(value)
 
-            url += '/{key}/{value}'.format(key=key, value=value)
+            url += f'/{key}/{value}'
 
         logger.debug(requests.utils.requote_uri(url))
 
@@ -427,8 +431,21 @@ class SpaceTrackClient(object):
 
     def _ratelimited_get(self, *args, **kwargs):
         """Perform get request, handling rate limiting."""
-        with self._ratelimiter:
-            resp = self.session.get(*args, **kwargs)
+        minute_limit = self._per_minute_throttle.check(RATELIMIT_KEY, 1)
+        hour_limit = self._per_hour_throttle.check(RATELIMIT_KEY, 1)
+
+        sleep_time = 0
+
+        if minute_limit.limited:
+            sleep_time = minute_limit.retry_after.total_seconds()
+
+        if hour_limit.limited:
+            sleep_time = max(sleep_time, hour_limit.retry_after.total_seconds())
+
+        if sleep_time > 0:
+            self._ratelimit_wait(sleep_time)
+
+        resp = self.session.get(*args, **kwargs)
 
         # It's possible that Space-Track will return HTTP status 500 with a
         # query rate limit violation. This can happen if a script is cancelled
@@ -440,19 +457,23 @@ class SpaceTrackClient(object):
             # Let's only retry if the error page tells us it's a rate limit
             # violation.
             if 'violated your query rate limit' in resp.text:
-                # Mimic the RateLimiter callback behaviour.
-                until = time.time() + self._ratelimiter.period
-                t = threading.Thread(target=self._ratelimit_callback, args=(until,))
-                t.daemon = True
-                t.start()
-                time.sleep(self._ratelimiter.period)
-
-                # Now retry
-                with self._ratelimiter:
-                    resp = self.session.get(*args, **kwargs)
+                # It seems that only the per-minute rate limit causes an HTTP
+                # 500 error. Breaking the per-hour limit seems to result in an
+                # email from Space-Track instead.
+                self._ratelimit_wait(
+                    self._per_minute_throttle.rate.period.total_seconds()
+                )
+                resp = self.session.get(*args, **kwargs)
 
         return resp
 
+    def _ratelimit_wait(self, duration):
+        until = time.monotonic() + duration
+        t = threading.Thread(target=self._ratelimit_callback, args=(until,))
+        t.daemon = True
+        t.start()
+        time.sleep(duration)
+
     def __getattr__(self, attr):
         if attr in self.request_controllers:
             controller_proxy = self._controller_proxies.get(attr)
@@ -465,8 +486,7 @@ class SpaceTrackClient(object):
             controller = self._find_controller(attr)
         except ValueError:
             raise AttributeError(
-                "'{name}' object has no attribute '{attr}'"
-                .format(name=self.__class__.__name__, attr=attr))
+                f"'{self.__class__.__name__}' object has no attribute '{attr}'")
 
         # generic_request can resolve the controller itself, but we
         # pass it because we have to check if the class_ is owned
@@ -500,7 +520,7 @@ class SpaceTrackClient(object):
             if class_ in classes:
                 return controller
         else:
-            raise ValueError('Unknown request class {!r}'.format(class_))
+            raise ValueError(f'Unknown request class {class_!r}')
 
     def _download_predicate_data(self, class_, controller):
         """Get raw predicate information for given request class, and cache for
@@ -508,8 +528,7 @@ class SpaceTrackClient(object):
         """
         self.authenticate()
 
-        url = ('{0}{1}/modeldef/class/{2}'
-               .format(self.base_url, controller, class_))
+        url = f'{self.base_url}{controller}/modeldef/class/{class_}'
 
         logger.debug(requests.utils.requote_uri(url))
 
@@ -530,10 +549,10 @@ class SpaceTrackClient(object):
                 classes = self.request_controllers.get(controller, None)
                 if classes is None:
                     raise ValueError(
-                        'Unknown request controller {!r}'.format(controller))
+                        f'Unknown request controller {controller!r}')
                 if class_ not in classes:
                     raise ValueError(
-                        'Unknown request class {!r}'.format(class_))
+                        f'Unknown request class {class_!r}')
 
             predicates_data = self._download_predicate_data(class_, controller)
             predicate_objects = self._parse_predicates_data(predicates_data)
@@ -548,7 +567,7 @@ class SpaceTrackClient(object):
             type_match = type_re.match(full_type)
             if not type_match:
                 raise ValueError(
-                    "Couldn't parse field type '{}'".format(full_type))
+                    f"Couldn't parse field type '{full_type}'")
 
             type_name = type_match.group(1)
             field_name = field['Field'].lower()
@@ -586,7 +605,7 @@ class SpaceTrackClient(object):
 
             if type_name not in types:
                 warnings.warn(
-                    'Unknown predicate type {!r}'.format(type_name),
+                    f'Unknown predicate type {type_name!r}',
                     UnknownPredicateTypeWarning,
                 )
 
@@ -600,7 +619,7 @@ class SpaceTrackClient(object):
                 enum_match = enum_re.match(full_type)
                 if not enum_match:
                     raise ValueError(
-                        "Couldn't parse enum type '{}'".format(full_type))
+                        f"Couldn't parse enum type '{full_type}'")
 
                 # match.groups() doesn't work for repeating groups, use findall
                 predicate.values = tuple(re.findall(r"'(\w+)'", full_type))
@@ -616,7 +635,7 @@ class SpaceTrackClient(object):
         return str(r)
 
 
-class _ControllerProxy(object):
+class _ControllerProxy:
     """Proxies request class methods with a preset request controller."""
     def __init__(self, client, controller):
         # The client will cache _ControllerProxy instances, so only store
@@ -626,9 +645,7 @@ class _ControllerProxy(object):
 
     def __getattr__(self, attr):
         if attr not in self.client.request_controllers[self.controller]:
-            raise AttributeError(
-                "'{self!r}' object has no attribute '{attr}'"
-                .format(self=self, attr=attr))
+            raise AttributeError(f"'{self!r}' object has no attribute '{attr}'")
 
         function = partial(
             self.client.generic_request, class_=attr,
@@ -692,8 +709,7 @@ def _iter_lines_generator(response, decode_unicode):
         else:
             pending = None
 
-        for line in lines:
-            yield line
+        yield from lines
 
     if pending is not None:
         yield pending
@@ -709,12 +725,16 @@ def _raise_for_status(response):
     http_error_msg = ''
 
     if 400 <= response.status_code < 500:
-        http_error_msg = '%s Client Error: %s for url: %s' % (
-            response.status_code, response.reason, response.url)
+        http_error_msg = (
+            f'{response.status_code} Client Error: {response.reason} '
+            f'for url: {response.url}'
+        )
 
     elif 500 <= response.status_code < 600:
-        http_error_msg = '%s Server Error: %s for url: %s' % (
-            response.status_code, response.reason, response.url)
+        http_error_msg = (
+            f'{response.status_code} Server Error: {response.reason} '
+            f'for url: {response.url}'
+        )
 
     if http_error_msg:
         spacetrack_error_msg = None
