@@ -1,29 +1,25 @@
-# coding: utf-8
-from __future__ import absolute_import, division, print_function
-
 import datetime as dt
 import json
+from unittest.mock import Mock, call, patch
 
 import pytest
 import requests
 import responses
 from requests import HTTPError, Response
+from rush.quota import Quota
 from spacetrack import (
     AuthenticationError, SpaceTrackClient, UnknownPredicateTypeWarning)
 from spacetrack.base import (
     Predicate, _iter_content_generator, _iter_lines_generator,
     _raise_for_status)
 
-try:
-    from unittest.mock import Mock, call, patch
-except ImportError:
-    from mock import Mock, call, patch
-
 
 def test_iter_lines_generator():
     """Test that lines are split correctly."""
     def mock_iter_content(self, chunk_size, decode_unicode):
-        for chunk in ['1\r\n2\r\n', '3\r', '\n4', '\r\n5']:
+        for chunk in [b'1\r\n2\r\n', b'3\r', b'\n4', b'\r\n5']:
+            if decode_unicode:
+                chunk = chunk.decode('utf-8')
             yield chunk
 
     with patch.object(Response, 'iter_content', mock_iter_content):
@@ -35,7 +31,9 @@ def test_iter_lines_generator():
 def test_iter_content_generator():
     """Test CRLF -> LF newline conversion."""
     def mock_iter_content(self, chunk_size, decode_unicode):
-        for chunk in ['1\r\n2\r\n', '3\r', '\n4', '\r\n5']:
+        for chunk in [b'1\r\n2\r\n', b'3\r', b'\n4', b'\r\n5']:
+            if decode_unicode:
+                chunk = chunk.decode('utf-8')
             yield chunk
 
     with patch.object(Response, 'iter_content', mock_iter_content):
@@ -45,7 +43,7 @@ def test_iter_content_generator():
 
         result = list(
             _iter_content_generator(response=Response(), decode_unicode=False))
-        assert result == ['1\r\n2\r\n', '3\r', '\n4', '\r\n5']
+        assert result == [b'1\r\n2\r\n', b'3\r', b'\n4', b'\r\n5']
 
 
 def test_generic_request_exceptions():
@@ -298,7 +296,7 @@ def test_ratelimit_error():
     st = SpaceTrackClient('identity', 'password')
 
     # Change ratelimiter period to speed up test
-    st._ratelimiter.period = 1
+    st._per_minute_throttle.rate = Quota.per_second(30)
 
     # Do it first without our own callback, then with.
 
@@ -356,7 +354,7 @@ def test_non_ratelimit_error():
     st = SpaceTrackClient('identity', 'password')
 
     # Change ratelimiter period to speed up test
-    st._ratelimiter.period = 1
+    st._per_minute_throttle.rate = Quota.per_second(30)
 
     mock_callback = Mock()
     st.callback = mock_callback
@@ -603,14 +601,16 @@ def test_dir():
     assert dir(st) == [
         '_authenticated',
         '_controller_proxies',
+        '_per_hour_throttle',
+        '_per_minute_throttle',
         '_predicates',
-        '_ratelimiter',
         'announcement',
         'base_url',
         'basicspacedata',
         'boxscore',
         'callback',
         'cdm',
+        'cdm_public',
         'decay',
         'delete',
         'download',
@@ -619,6 +619,7 @@ def test_dir():
         'file_history',
         'fileshare',
         'gp',
+        'gp_history',
         'identity',
         'launch_site',
         'maneuver',
